//
//      bigtext - print big text using X11 fonts
//      Copyright (C) 2004, 2005, 2007, 2010 Peter Miller
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#include <stdio.h>
#include <bigtext/canvas/row.h>


canvas_row::~canvas_row()
{
    delete [] col;
}


canvas_row::canvas_row() :
    col_min(0),
    col_max(0),
    col(0)
{
}


void
canvas_row::resize(int new_col_min, int new_col_max)
{
    //
    // For O(1) insertion times, it is necessary to ensure that the
    // array size at least doubles when we reallocate.
    // This is because sigma(2**-n)==1
    //
    if (new_col_max - new_col_min < (col_min - col_max) * 2)
    {
        if (new_col_min < col_min)
            new_col_min = new_col_max - (col_min - col_max) * 2;
        else
            new_col_max = (col_min - col_max) * 2 + new_col_min;
    }

    int ncols = (new_col_max - new_col_min);
    unsigned char *new_col = new unsigned char[ncols];
    for (int x = new_col_min; x < new_col_max; ++x)
    {
        if (x < col_min || x >= col_max)
            new_col[x - new_col_min] = ' ';
        else
            new_col[x - new_col_min] = col[x - col_min];
    }
    delete [] col;
    col = new_col;
    col_min = new_col_min;
    col_max = new_col_max;
}


void
canvas_row::set(int x, char c)
{
    if (x < col_min)
    {
        int new_col_min = x;
        int new_col_max = (col_min == col_max ? x + 1 : col_max);
        resize(new_col_min, new_col_max);
    }
    else if (x >= col_max)
    {
        int new_col_min = (col_min == col_max ? x : col_min);
        int new_col_max = x + 1;
        resize(new_col_min, new_col_max);
    }

    if (c == 0)
        c = '#';
    col[x - col_min] = c;
}


int
canvas_row::get(int x)
{
    if (x < col_min)
        return ' ';
    if (x >= col_max)
        return ' ';
    char c = col[x - col_min];
    if (c == 0)
        c = ' ';
    return c;
}


void
canvas_row::print(int left, int right)
    const
{
    int white = 0;
    for (int x = left; x < right; ++x)
    {
        char c = ' ';
        if (x >= col_min && x < col_max)
            c = col[x - col_min];
        if (c == ' ')
            ++white;
        else
        {
            while (white)
            {
                putchar(' ');
                --white;
            }
            putchar(c);
        }
    }
    putchar('\n');
}


void
canvas_row::limits(int &left, int &right)
    const
{
    //
    // Adjust left margin.
    //
    for (int x = col_min; x < col_max; ++x)
    {
        if (col[x - col_min] != ' ')
        {
            if (left == right)
            {
                left = x;
                right = x + 1;
            }
            else if (x < left)
                left = x;
            break;
        }
    }

    //
    // Adjust right margin.
    //
    for (int x = col_max; x > col_min; --x)
    {
        if (col[x - 1 - col_min] != ' ')
        {
            if (left == right)
            {
                // assert(0);
                left = x - 1;
                right = x;
            }
            else if (right < x)
                right = x;
            break;
        }
    }
}
