//
// bigtext - print big text using X11 fonts
// Copyright (C) 2004, 2007-2010 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <cstdio>
#include <cstring>
#include <string>

#include <bigtext/font.h>
#include <bigtext/glyph.h>


font::~font()
{
}


font::font()
{
}


static std::string
field(const std::string &base, char sep, int n)
{
    const char *s = base.c_str();
    while (n > 0)
    {
        const char *cp = strchr(s, sep);
        if (!cp)
            return "";
        --n;
        s = cp + 1;
    }
    const char *cp = strchr(s, sep);
    if (!cp)
        return s;
    return std::string(s, cp - s);
}


static int
minuses(const char *s)
{
    int result = 0;
    while (s)
    {
        const char *cp = strchr(s, '-');
        if (!cp)
            break;
        ++result;
        s = cp + 1;
    }
    return result;
}


static std::string
mangle(const std::string &name)
{
    std::string bname(name);
    if (minuses(name.c_str()) > 8)
    {
        std::string typeface = field(name, '-', 2).c_str();
        std::string weight = field(name, '-', 3).c_str();
        if (weight == "bold" || weight == "demibold" || weight == "demi bold")
            weight = "_bold";
        else
            weight = "";
        std::string slant = field(name, '-', 4).c_str();
        if (slant == "i" || slant == "o")
            slant = "_italic";
        else
            slant = "";
        std::string size = field(name, '-', 7).c_str();
        bname = typeface + weight + slant + "-" + size;
    }

    char result[100];
    char *cp = result;
    char *end = result + sizeof(result) - 1;
    const char *np = bname.c_str();
    while (*np && cp < end)
    {
        unsigned char c = *np++;
        switch (c)
        {
        case 'a': case 'b': case 'c': case 'd': case 'e': case 'f': case 'g':
        case 'h': case 'i': case 'j': case 'k': case 'l': case 'm': case 'n':
        case 'o': case 'p': case 'q': case 'r': case 's': case 't': case 'u':
        case 'v': case 'w': case 'x': case 'y': case 'z':
        case 'A': case 'B': case 'C': case 'D': case 'E': case 'F': case 'G':
        case 'H': case 'I': case 'J': case 'K': case 'L': case 'M': case 'N':
        case 'O': case 'P': case 'Q': case 'R': case 'S': case 'T': case 'U':
        case 'V': case 'W': case 'X': case 'Y': case 'Z':
        case '0': case '1': case '2': case '3': case '4':
        case '5': case '6': case '7': case '8': case '9':
            *cp++ = c;
            break;

        default:
            if (cp > result && cp[-1] != '_')
                *cp++ = '_';
            break;
        }
    }
    while (cp > result && cp[-1] == '_')
        --cp;
    if (cp == result)
        return "noname";
    return std::string(result, cp - result);
}


void
font::print_internal_header()
{
    std::string c_name = mangle(get_name());
    const char *name = c_name.c_str();

    printf("#ifndef FONT_INTERNAL_%s_H\n", name);
    printf("#define FONT_INTERNAL_%s_H\n", name);
    printf("\n");
    printf("#pragma interface \"font_internal_%s\"\n", name);
    printf("\n");
    printf("#include <font/internal.h>\n");
    printf("\n");
    printf("/**\n");
    printf("  * The font_internal_%s class is used to represent the\n", name);
    printf("  * built-in form of the \"%s\" font.\n", get_name());
    printf("  */\n");
    printf("class font_internal_%s:\n", name);
    printf("    public font_internal\n");
    printf("{\n");
    printf("public:\n");
    printf("    /**\n");
    printf("      * The destructor.\n");
    printf("      */\n");
    printf("    virtual ~font_internal_%s();\n", name);
    printf("\n");
    printf("    /**\n");
    printf("      * The default constructor.\n");
    printf("      */\n");
    printf("    font_internal_%s();\n", name);
    printf("\n");
    printf("    // See base class for documentation.\n");
    printf("    glyph *get_glyph(unsigned n);\n");
    printf("\n");
    printf("    // See base class for documentation.\n");
    printf("    int get_ascent();\n");
    printf("\n");
    printf("    // See base class for documentation.\n");
    printf("    int get_descent();\n");
    printf("\n");
    printf("    // See base class for documentation.\n");
    printf("    const char *get_name() const;\n");
    printf("\n");
    printf("private:\n");
    printf("    /**\n");
    printf("      * The glyph_ptr instance variable is used to\n");
    printf("      * remember the pointers to instances of the\n");
    printf("      * glyphs which make up the font.\n");
    printf("      */\n");
    printf("    glyph **glyph_ptr;\n");
    printf("\n");
    printf("    /**\n");
    printf("      * The copy constructor.  Do not use.\n");
    printf("      */\n");
    printf("    font_internal_%s(const font_internal_%s &);\n", name, name);
    printf("\n");
    printf("    /**\n");
    printf("      * The assignment operator.  Do not use.\n");
    printf("      */\n");
    printf("    font_internal_%s &operator=(const font_internal_%s &);\n",
        name, name);
    printf("};\n");
    printf("\n");
    printf("#endif // FONT_INTERNAL_%s_H\n", name);
}


void
font::print_internal_code()
{
    const char *long_name = get_name();
    std::string c_name = mangle(long_name);
    const char *name = c_name.c_str();

    printf("#pragma implementation \"font_internal_%s\"\n", name);
    printf("\n");
    printf("#include <font/internal/%s.h>\n", name);
    printf("#include <glyph/internal.h>\n");
    printf("\n");
    printf("static glyph_internal::info_t table[] =\n{\n");
    for (int j = 0; j < 256; ++j)
    {
        glyph *gp = get_glyph(j);
        gp->print_internal_definition();
    }
    printf("};\n");

    printf("\n");
    printf("font_internal_%s::~font_internal_%s()\n", name, name);
    printf("{\n");
    printf("    if (glyph_ptr)\n");
    printf("    {\n");
    printf("        for (int j = 0; j < 256; ++j)\n");
    printf("            delete glyph_ptr[j];\n");
    printf("        delete [] glyph_ptr;\n");
    printf("    }\n");
    printf("}\n");

    printf("\n");
    printf("font_internal_%s::font_internal_%s() :\n", name, name);
    printf("    glyph_ptr(0)\n");
    printf("{\n");
    printf("}\n");

    printf("\n");
    printf("glyph *\n");
    printf("font_internal_%s::get_glyph(unsigned n)\n", name);
    printf("{\n");
    printf("    if (n > 256)\n");
    printf("        n = '?';\n");
    printf("    if (!glyph_ptr)\n");
    printf("    {\n");
    printf("        glyph_ptr = new glyph * [256];\n");
    printf("        for (unsigned j = 0; j < 256; ++j)\n");
    printf("            glyph_ptr[j] = 0;\n");
    printf("    }\n");
    printf("    if (!glyph_ptr[n])\n");
    printf("        glyph_ptr[n] = new glyph_internal(table[n]);\n");
    printf("    return glyph_ptr[n];\n");
    printf("}\n");

    printf("\n");
    printf("int\n");
    printf("font_internal_%s::get_ascent()\n", name);
    printf("{\n");
    printf("    return %d;\n", get_ascent());
    printf("}\n");

    printf("\n");
    printf("int\n");
    printf("font_internal_%s::get_descent()\n", name);
    printf("{\n");
    printf("    return %d;\n", get_descent());
    printf("}\n");

    printf("\n");
    printf("const char *\n");
    printf("font_internal_%s::get_name()\n", name);
    printf("    const\n");
    printf("{\n");
    printf("    return \"%s\";\n", long_name);
    printf("}\n");
}


void
font::print_apple_pascal()
{
    printf("#\n");
    printf("# Derived from the X11 \"%s\" font.\n", get_name());
    printf("#\n");
    printf("#   This file defines the shapes of the characters for\n");
    printf("#   drawing them on the screen.  This what is known as\n");
    printf("#   a bit-mapped font.\n");
    printf("#\n");
    printf("#   Note that font outlines and bit-mapped fonts are\n");
    printf("#   not eligible for copyright protection.  Only fonts\n");
    printf("#   which contain computer code (e.g. some PostScript\n");
    printf("#   fonts) are eligible for copyright protection.\n");
    printf("#\n");
    for (int j = 0; j < 256; ++j)
    {
        if (j == '\'' || j == '\\')
            printf("'\\%c'", j);
        else if (isprint(j))
            printf("'%c'", j);
        else
            printf("%d", j);
        glyph *gp = get_glyph(j);
        gp->print_apple_pascal(get_descent(), get_ascent());
    }
}
