//
//      bigtext - print big text using X11 fonts
//      Copyright (C) 2004, 2007, 2010 Peter Miller
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef BIGTEXT_FONT_H
#define BIGTEXT_FONT_H

#include <X11/fonts/FSlib.h>

class font_server; // forward
class glyph; // forward


/**
  * The font class is an abstract interface to a font.  It can be used
  * to obtain individual glyphs which are used to actually draw each
  * character.
  */
class font
{
public:
    /**
      * The destructor.
      */
    virtual ~font();

protected:
    /**
      * The default constructor.
      * It may only be called from derived classes.
      */
    font();

public:
    /**
      * The get_glyph method is used to get the glyph corresponding to
      * the given character.
      *
      * DO NOT delete this value when you are done with it,
      * that is the font's job.
      */
    virtual glyph *get_glyph(unsigned c) = 0;

    /**
      * The get_ascent method is used to get the ascent above the
      * baseline for the whole font.  (Usually, this means the height of
      * the letter 'T'.)
      */
    virtual int get_ascent() = 0;

    /**
      * The get_descent method is used to get the descent below the
      * baseline for the whole font.  (Usually, this means the depth of
      * the letter 'y'.)
      */
    virtual int get_descent() = 0;

    /**
      * The get_name method is used to get the name of the font.
      */
    virtual const char *get_name() const = 0;

    /**
      * The print_apple_pascal method is used to print code suitable for
      * building a SYSTEM.CHARSET file for Apple ][ Pascal.
      */
    void print_apple_pascal();

    /**
      * The print_internal_header method is used to print code suitable
      * for forming an internal font defintion.  This is the class
      * interface.
      */
    void print_internal_header();

    /**
      * The print_internal_code method is used to print code
      * suitable for forming an internal font defintion.
      * This is the class implementation.
      */
    void print_internal_code();

private:
    /**
      * The copy constructor.  Do not use.
      */
    font(const font &);

    /**
      * The assignment operator.  Do not use.
      */
    font &operator=(const font &);
};

#endif // BIGTEXT_FONT_H
