//
//      bigtext - print big text using X11 fonts
//      Copyright (C) 2004, 2007, 2010 Peter Miller
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef BIGTEXT_FONT_XFS_H
#define BIGTEXT_FONT_XFS_H

#include <bigtext/font.h>
#include <bigtext/font_server/xfs.h>

/**
  * The font_xfs class is used to represent the state of a font obtained
  * from the X11 Font Server.
  */
class font_xfs:
    public font
{
public:
    /**
      * The destructor.
      */
    virtual ~font_xfs();

    /**
      * The constructor.
      */
    font_xfs(font_server_xfs *, const char *name);

    // See base class for documentation.
    glyph *get_glyph(unsigned c);

    // See base class for documentation.
    int get_ascent();

    // See base class for documentation.
    int get_descent();

    // See base class for documentation.
    const char *get_name() const;

    /**
      * The query_x_extents_8 method is called by the glyph_xfs::get_info
      * method to obtain information about an individual character.  It
      * should never be called by any other client.
      *
      * \param c
      *     The character to get information about.
      * \param info
      *     Where to place the information.
      */
    void query_x_extents_8(unsigned char c, FSXCharInfo *info);

    /**
      * The query_x_bitmaps_8 method is called by the glyph_xfs::get_info
      * method to obtain the bitmap for an individual character.  It
      * should never be called by any other client.
      *
      * \param c
      *     The character to get the bitmap for.
      * \param bitmap
      *     Where to place the bitmap.
      *
      * \note
      *     Use FSFree on the returned bitmap when you are done with it.
      */
    void query_x_bitmaps_8(unsigned char c, unsigned char **bitmap);

private:
    /**
      * The server instance variable is used to remember the font_xfs server
      * we can direct informational queries to.
      */
    font_server_xfs *server;

    /**
      * The name instance variable is used to remember the name of this
      * font_xfs.
      */
    const char *name;

    /**
      * The id instance variable is used to remember the font_xfs ID of this
      * font_xfs, as given to us by the font_xfs server.  It has to be used when
      * querying the font_xfs server about this font_xfs.
      */
    int id;

    /**
      * The info_header instance variable is used to remember
      * information about the whole font_xfs (as opposed to individual
      * glyphs).
      */
    FSXFontInfoHeader info_header;

    /**
      * The min_char instance variable is used to remember the minimum
      * character value of this font_xfs.  Characters less then this will
      * not display as expected.
      */
    unsigned min_char;

    /**
      * The max_char instance variable is used to remember the maximum
      * character value of this font_xfs.  Characters more then this will
      * not display as expected.
      */
    unsigned max_char;

    /**
      * The glyphs instance variable is used to remember the locations
      * of glyphs.  It is indexed by character values.
      */
    glyph *glyphs[256];

    /**
      * The get_id method is used to get the font_xfs ID from the font_xfs
      * server, and also any other information about the font_xfs.
      * The results are cached.
      */
    Font get_id();

    /**
      * The default constructor.  Do not use.
      */
    font_xfs();

    /**
      * The copy constructor.  Do not use.
      */
    font_xfs(const font_xfs &);

    /**
      * The assignment operator.  Do not use.
      */
    font_xfs &operator=(const font_xfs &);
};

#endif // BIGTEXT_FONT_XFS_H
