//
// bigtext - print big text using X11 fonts
// Copyright (C) 2004, 2007, 2009, 2010, 2012 Peter Miller
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 3 of the License, or (at
// your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program. If not, see <http://www.gnu.org/licenses/>.
//

#include <lib/config.h>
#include <cerrno>
#include <cstdarg>
#include <cstdio>
#include <cstdlib>
#include <cstring>
#include <libexplain/output.h>

#include <bigtext/font/xfs.h>
#include <bigtext/font_server/xfs.h>


font_server_xfs::~font_server_xfs()
{
    if (server)
    {
        FSCloseServer(server);
        server = 0;
    }
}


font_server_xfs::font_server_xfs() :
    server_name(0),
    server(0)
{
}


const char *
font_server_xfs::get_server_name(void)
{
    if (!server_name)
    {
        server_name = getenv("FONTSERVER");
        if (!server_name)
        {
            // FIXME: ask X11 what the FontPath is.
            server_name = "unix/:7100";
        }
    }
    return server_name;
}


FSServer *
font_server_xfs::get_server(void)
{
    if (!server)
    {
        // the cast is because the prototype is brain-dead
        errno = EINVAL;
        server = FSOpenServer((char *)get_server_name());
        if (!server)
        {
            int err = errno;
            //
            // The error is most likely in connect(2), but we can't know
            // for sure, so we pike, and use strerror.
            //
            explain_output_error_and_die
            (
                "Can't open font server \"%s\": %s\n",
                get_server_name(),
                strerror(err)
            );
        }
    }
    return server;
}


font_list
font_server_xfs::get_font_list(const char *pattern)
{
    if (!pattern || !*pattern)
        pattern = "*";
    int count;
    // The cast is because the prototype is brain-dead.
    char **list = FSListFonts(get_server(), (char *)pattern, 5000, &count);
    if (!list)
    {
        fprintf(stderr, "FSListFonts failed\n");
        exit(1);
    }
    font_list result((const char **)list, count);
    FSFreeFontNames(list);
    return result;
}


static FSBitmapFormat
get_format(void)
{
    return (BitmapFormatByteOrderMSB |
        BitmapFormatBitOrderMSB | BitmapFormatImageRectMin |
        BitmapFormatScanlinePad8 | BitmapFormatScanlineUnit8);
}


static FSBitmapFormatMask
get_format_mask(void)
{
    return (BitmapFormatMaskByte |
        BitmapFormatMaskBit | BitmapFormatMaskImageRectangle |
        BitmapFormatMaskScanLinePad | BitmapFormatMaskScanLineUnit);
}


font *
font_server_xfs::get_font(const char *name)
{
    return new font_xfs(this, name);
}


Font
font_server_xfs::open_bitmap_font(const char *name)
{
    Font other_id;
    Font fid =
        FSOpenBitmapFont
        (
            get_server(),
            get_format(),
            get_format_mask(),
            (char *)name,
            &other_id
        );
    if (fid == 0)
    {
        fprintf(stderr, "FSOpenBitmapFont(\"%s\") failed\n", name);
        exit(1);
    }
    return fid;
}


void
font_server_xfs::close_font(Font fid)
{
    FSCloseFont(get_server(), fid);
}


void
font_server_xfs::fatal_error(int code, const char *fmt, ...)
{
    va_list ap;
    va_start(ap, fmt);
    char buffer1[1000];
    vsnprintf(buffer1, sizeof(buffer1), fmt, ap);
    va_end(ap);

    char buffer2[1000];
    FSGetErrorText(get_server(), code, buffer2, sizeof(buffer2));

    fprintf(stderr, "%s: %s\n", buffer1, buffer2);
    exit(1);
}


void
font_server_xfs::query_x_info(Font fid, FSXFontInfoHeader *info,
    FSPropInfo *props, FSPropOffset **offsets, unsigned char **prop_data)
{
    int code = FSQueryXInfo(get_server(), fid, info, props, offsets, prop_data);
    if (code != FSSuccess)
        fatal_error(code, "FSQueryXInfo");
}


void
font_server_xfs::query_x_extents_8(Font fid, unsigned char *str,
    unsigned long str_len, FSXCharInfo **extents)
{
    int range_type = False;
    int code =
        FSQueryXExtents8(get_server(), fid, range_type, str, str_len, extents);
    if (code != FSSuccess)
        fatal_error(code, "FSQueryXExtents8");
}


void
font_server_xfs::query_x_bitmaps_8(Font fid, unsigned char *str,
    unsigned long str_len, FSOffset **offsets, unsigned char **glyphdata)
{
    int range_type = False;
    int code =
        FSQueryXBitmaps8
        (
            get_server(),
            fid,
            get_format(),
            range_type,
            str,
            str_len,
            offsets,
            glyphdata
        );
    if (code != FSSuccess)
        fatal_error(code, "FSQueryXBitmaps8");
}


// vim: set ts=8 sw=4 et :
