//
//      bigtext - print big text using X11 fonts
//      Copyright (C) 2004, 2007, 2010 Peter Miller
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef BIGTEXT_FONT_SERVER_XFS_H
#define BIGTEXT_FONT_SERVER_XFS_H

#include <X11/fonts/FSlib.h>

#include <bigtext/font_server.h>

/**
  * The font_server_xfs class is used to wrap the interface to the X11 Font
  * Server.  It takes care of openning rthe connection when it's needed,
  * and closing it when we are done.
  */
class font_server_xfs:
    public font_server
{
public:
    /**
      * The destructor.
      */
    virtual ~font_server_xfs();

    /**
      * The default constructor.
      */
    font_server_xfs();

    // See base class for documentation.
    font_list get_font_list(const char *pattern = 0);

    // See base class for documentation.
    font *get_font(const char *name);

    /**
      * The open_bitmap_font method is used to query the font server
      * (via the FSOpenBitmapFont function).  It shall only be called
      * from the font::get_id method.
      */
    Font open_bitmap_font(const char *name);

    /**
      * The close_font method is used to tell the font server we are
      * done with a font (via the FSCloseFont function).  It shall only
      * be called from the font class.
      */
    void close_font(Font fid);

    /**
      * The query_x_info method is used to obtain information about a
      * font.  It shall only be called by the font class.
      */
    void query_x_info(Font fid, FSXFontInfoHeader *info, FSPropInfo *props,
        FSPropOffset **offsets, unsigned char **prop_data);

    /**
      * The query_x_extents_8 method is used to obtain information about
      * a glyph.  It shall only be called by the font class.
      */
    void query_x_extents_8(Font fid, unsigned char *str, unsigned long str_len,
        FSXCharInfo **extents);

    /**
      * The query_x_bitmaps_8 method is used to obtain the bitmap for a
      * glyph.  It shall only be called by the font class.
      */
    void query_x_bitmaps_8(Font fid, unsigned char *str, unsigned long str_len,
        FSOffset **offsets, unsigned char **glyphdata);

private:
    /**
      * The get_server_name method is used to get the server name to
      * contect.  This is deferred and cached.
      */
    const char *get_server_name();

    /**
      * The get_server method is used to get the server to connect to.
      * This is deferred and cahced.
      */
    FSServer *get_server();

    /**
      * The fatal_error method is used to report a fatal error.  It does
      * not return.
      */
    void fatal_error(int, const char *, ...);

    /**
      * The server_name instance variable is used to remember the name
      * of the font server to talk to.
      */
    const char *server_name;

    /**
      * The server instance variable is used to remember the server we
      * are connected to.  It set to NULL if we are not yet connected.
      */
    FSServer *server;

    /**
      * The copy constructor.  Do not use.
      */
    font_server_xfs(const font_server_xfs &);

    /**
      * The assignment operator.  Do not use.
      */
    font_server_xfs &operator=(const font_server_xfs &);
};

#endif // BIGTEXT_FONT_SERVER_XFS_H
