//
//      bigtext - print big text using X11 fonts
//      Copyright (C) 2004, 2005, 2007, 2010 Peter Miller
//
//      This program is free software; you can redistribute it and/or modify
//      it under the terms of the GNU General Public License as published by
//      the Free Software Foundation; either version 3 of the License, or
//      (at your option) any later version.
//
//      This program is distributed in the hope that it will be useful,
//      but WITHOUT ANY WARRANTY; without even the implied warranty of
//      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//      GNU General Public License for more details.
//
//      You should have received a copy of the GNU General Public License
//      along with this program. If not, see
//      <http://www.gnu.org/licenses/>.
//

#ifndef BIGTEXT_GRAPHICS_CONTEXT_H
#define BIGTEXT_GRAPHICS_CONTEXT_H

class canvas; // forward
class font; // forward
class font_server; // forward

/**
  * The graphics_context class is used to represent the present stat of
  * the graphics drawing state, including the font and cursor position.
  */
class graphics_context
{
public:
    /**
      * The destructor.
      */
    virtual ~graphics_context();

    /**
      * The constructor.
      */
    graphics_context(canvas &, font_server &);

    /**
      * The set_font method is used to set the font to be used to show
      * text.  Note that this is in the X11 style, and thus include
      * the typeface name (e.g. TimesRoman) the style (e.g. normal or
      * italic) and the point size.
      */
    void set_font(const char *name);

    /**
      * The show method is used to draw a string at the current cursor
      * position.  A newline will advance the cursor position down the
      * page and return the X position to 0.  Other characters will
      * advance the cursor to the right.
      */
    void show(const char *text);

    /**
      * The set_self_char method is used to specify that characters
      * should draw each pixel of themselves as themselves, rather than
      * the default character.  This may get wiped every time you change
      * fonts.
      */
    void set_self_char();

    /**
      * The set_pen method is used to specify the constant character
      * used to draw the pixels of all characters.
      */
    void set_pen(int c);

private:
    /**
      * The drawable instance variable is used to remember the canvast
      * this graphics context is drawing on.
      */
    canvas &drawable;

    /**
      * The server instance variable is used to remember which font
      * server this graphics context is using to draw chacatres.
      */
    font_server &server;

    /**
      * The the_font instance variable is used to remember the current
      * font.  Set to NULL if not font is currently selected.
      */
    font *the_font;

    /**
      * The cursor_x instance variable is used to remember the current
      * drawing point X coordinate.
      */
    int cursor_x;

    /**
      * The cursor_y instance variable is used to remember the current
      * drawing point Y coordinate.
      */
    int cursor_y;

    /**
      * The pen instance variable is used to remember the "pen" being
      * drawn with at the moment.  This is usually a specific character
      * to be used to draw each pixel of each glyph.  The special value
      * -1 means to draw each character as itself.
      */
    int pen;

    /**
      * The default constructor.
      */
    graphics_context();

    /**
      * The copy constructor.
      */
    graphics_context(const graphics_context &);

    /**
      * The assignment operator.
      */
    graphics_context &operator=(const graphics_context &);
};

#endif // BIGTEXT_GRAPHICS_CONTEXT_H
